==> nonebot_plugin_orm-0.2.0-py3-none-any.whl: nonebot_plugin_orm/model.py <==
from __future__ import annotations

import sys
from inspect import Parameter, Signature
from typing import TYPE_CHECKING, Any, ClassVar
from typing_extensions import Self, Unpack, Annotated

from nonebot.params import Depends
from nonebot import get_plugin_by_module_name
from sqlalchemy import Table, MetaData, select
from pydantic.typing import get_args, get_origin
from sqlalchemy.orm import Mapped, DeclarativeBase

from .utils import DependsInner, get_annotations

if sys.version_info >= (3, 9):
    from typing import Annotated
else:
    from typing_extensions import Annotated

__all__ = ("Model",)


NAMING_CONVENTION = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}


class Model(DeclarativeBase):
    metadata = MetaData(naming_convention=NAMING_CONVENTION)

    if TYPE_CHECKING:
        __args__: ClassVar[tuple[type[Self], Unpack[tuple[Any, ...]]]]
        __origin__: type[Annotated]

        __table__: ClassVar[Table]
        __bind_key__: ClassVar[str]

    def __init_subclass__(cls) -> None:
        _setup_di(cls)
        _setup_tablename(cls)

        super().__init_subclass__()

        if not hasattr(cls, "__table__"):
            return

        _setup_bind(cls)


def _setup_di(cls: type[Model]) -> None:
    """Get signature for NoneBot's dependency injection,
    and set annotations for SQLAlchemy declarative class.
    """
    from . import async_scoped_session

    parameters: list[Parameter] = [
        Parameter(
            "__session__", Parameter.KEYWORD_ONLY, annotation=async_scoped_session
        )
    ]

    annotations: dict[str, Any] = {}
    for base in reversed(cls.__mro__):
        annotations.update(get_annotations(base, eval_str=True))

    for name, type_annotation in annotations.items():
        # Check if the attribute is both a dependent and a mapped column
        depends_inner = None
        if get_origin(type_annotation) is Annotated:
            (type_annotation, *extra_args) = get_args(type_annotation)
            depends_inner = next(
                (x for x in extra_args if isinstance(x, DependsInner)), None
            )

        if get_origin(type_annotation) is not Mapped:
            continue

        default = getattr(cls, name, Signature.empty)

        depends_inner = default if isinstance(default, DependsInner) else depends_inner
        if depends_inner is None:
            continue

        # Set parameter for NoneBot dependency injection
        parameters.append(
            Parameter(
                name,
                Parameter.KEYWORD_ONLY,
                default=depends_inner,
                annotation=get_args(type_annotation)[0],
            )
        )

        # Set annotation for SQLAlchemy declarative class
        cls.__annotations__[name] = type_annotation
        if default is not Signature.empty and not isinstance(default, Mapped):
            delattr(cls, name)

    async def dependency(
        *, __session__: async_scoped_session, **kwargs: Any
    ) -> Model | None:
        return await __session__.scalar(select(cls).filter_by(**kwargs))

    dependency.__signature__ = Signature(parameters)
    cls.__args__ = (Model, Depends(dependency))
    cls.__origin__ = Annotated


def _setup_tablename(cls: type[Model]) -> None:
    for attr in ("__abstract__", "__tablename__", "__table__"):
        if getattr(cls, attr, None):
            return

    cls.__tablename__ = cls.__name__.lower()

    if plugin := get_plugin_by_module_name(cls.__module__):
        cls.__tablename__ = f"{plugin.name.replace('-', '_')}_{cls.__tablename__}"


def _setup_bind(cls: type[Model]) -> None:
    bind_key: str | None = getattr(cls, "__bind_key__", None)

    if bind_key is None:
        if plugin := get_plugin_by_module_name(cls.__module__):
            bind_key = plugin.name
        else:
            bind_key = ""

    cls.__table__.info["bind_key"] = bind_key

==> nonebot_plugin_orm-0.2.4-py3-none-any.whl: nonebot_plugin_orm/model.py <==
from __future__ import annotations

import sys
from inspect import Parameter, Signature
from typing import TYPE_CHECKING, Any, ClassVar

from nonebot.params import Depends
from nonebot import get_plugin_by_module_name
from sqlalchemy import Table, MetaData, select
from pydantic.typing import get_args, get_origin
from sqlalchemy.orm import Mapped, DeclarativeBase
from sqlalchemy.orm.decl_api import DeclarativeAttributeIntercept

from .utils import DependsInner, get_annotations

if sys.version_info >= (3, 9):
    from typing import Annotated
else:
    from typing_extensions import Annotated

__all__ = ("Model",)


NAMING_CONVENTION = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}


class ModelMeta(DeclarativeAttributeIntercept):
    if TYPE_CHECKING:
        __signature__: Signature

    def __new__(
        mcs,
        name: str,
        bases: tuple[type, ...],
        namespace: dict[str, Any],
        **kwargs: Any,
    ) -> ModelMeta:
        from . import async_scoped_session

        cls: ModelMeta = super().__new__(mcs, name, bases, namespace, **kwargs)

        if not (signature := getattr(cls, "__signature__", None)):
            return cls

        async def dependency(
            *, __session: async_scoped_session, **kwargs: Any
        ) -> ModelMeta | None:
            return await __session.scalar(select(cls).filter_by(**kwargs))

        dependency.__signature__ = Signature(
            (
                Parameter(
                    "_ModelMeta__session",
                    Parameter.KEYWORD_ONLY,
                    annotation=async_scoped_session,
                ),
                *signature.parameters.values(),
            )
        )

        return Annotated[cls, Depends(dependency)]


class Model(DeclarativeBase, metaclass=ModelMeta):
    metadata = MetaData(naming_convention=NAMING_CONVENTION)

    if TYPE_CHECKING:
        __table__: ClassVar[Table]
        __bind_key__: ClassVar[str]

    def __init_subclass__(cls) -> None:
        _setup_di(cls)
        _setup_tablename(cls)

        super().__init_subclass__()

        if not hasattr(cls, "__table__"):
            return

        _setup_bind(cls)


def _setup_di(cls: type[Model]) -> None:
    """Get signature for NoneBot's dependency injection,
    and set annotations for SQLAlchemy declarative class.
    """
    from . import async_scoped_session

    parameters: list[Parameter] = []

    annotations: dict[str, Any] = {}
    for base in reversed(cls.__mro__):
        annotations.update(get_annotations(base, eval_str=True))

    for name, type_annotation in annotations.items():
        # Check if the attribute is both a dependent and a mapped column
        depends_inner = None
        if get_origin(type_annotation) is Annotated:
            (type_annotation, *extra_args) = get_args(type_annotation)
            depends_inner = next(
                (x for x in extra_args if isinstance(x, DependsInner)), None
            )

        if get_origin(type_annotation) is not Mapped:
            continue

        default = getattr(cls, name, Signature.empty)

        depends_inner = default if isinstance(default, DependsInner) else depends_inner
        if depends_inner is None:
            continue

        # Set parameter for NoneBot dependency injection
        parameters.append(
            Parameter(
                name,
                Parameter.KEYWORD_ONLY,
                default=depends_inner,
                annotation=get_args(type_annotation)[0],
            )
        )

        # Set annotation for SQLAlchemy declarative class
        cls.__annotations__[name] = type_annotation
        if default is not Signature.empty and not isinstance(default, Mapped):
            delattr(cls, name)

    cls.__signature__ = Signature(parameters)


def _setup_tablename(cls: type[Model]) -> None:
    for attr in ("__abstract__", "__tablename__", "__table__"):
        if getattr(cls, attr, None):
            return

    cls.__tablename__ = cls.__name__.lower()

    if plugin := get_plugin_by_module_name(cls.__module__):
        cls.__tablename__ = f"{plugin.name.replace('-', '_')}_{cls.__tablename__}"


def _setup_bind(cls: type[Model]) -> None:
    bind_key: str | None = getattr(cls, "__bind_key__", None)

    if bind_key is None:
        if plugin := get_plugin_by_module_name(cls.__module__):
            bind_key = plugin.name
        else:
            bind_key = ""

    cls.__table__.info["bind_key"] = bind_key
